package dao

import model.TransactionRequest
import java.sql.DriverManager
import java.sql.ResultSet
import java.util.Properties

class TransactionDao {
    private val transactionRequestQuery = """
        select txn_no,
           trim(src_ac_no) AS src_ac_no,
           trim(dest_ac_no) AS dest_ac_no,
           ifsc_code,
           txn_amt,
           txn_date,
           t.teller_id,
           (case
             when t.ifsc_code like 'WBSC%' then
              'FAILED'
             else
              'RECEIVED'
           end) AS status,
           substr(regexp_replace(regexp_replace(upper(beneficiary_name),'[^A-Z0-9 ]',''),' {2,}',' '),1,35) AS beneficiary_name,
           beneficiary_add,
           t.pacs_id,
           dccb_code,
           TO_NUMBER(cbs_br_code) AS br_code,
           substr(regexp_replace(regexp_replace(upper(remm_name),'[^A-Z0-9 ]',''),' {2,}',' '),1,35) AS remitter_name,
           ipks_accno AS pacs_acc_no,
           da.link_accno AS cbs_sb_acc_no,
           'pacs_db' AS db_name
      from neft_rtgs_txn t
      join dep_account da ON t.ipks_accno = da.key_1
     where 
       t.txn_date = (select system_date from system_date)
       and t.STATUS = 'A'
       and t.bank_channel = 'SCB'
       and da.link_accno IS NOT NULL
    """.trimIndent()

    fun getTransactionRequests(): List<TransactionRequest> {
        val prop = loadProp()

        val dbHost = getProp(prop, "DB_HOST")
        val dbPort = getProp(prop, "DB_PORT")
        val dbName = getProp(prop, "DB_NAME")

        val dbUrl = "jdbc:oracle:thin:@$dbHost:$dbPort:$dbName"

        val dbUser = getProp(prop, "DB_USER")
        val dbPassword = getProp(prop, "DB_PASSWORD")

        val transactionList: List<TransactionRequest>

        DriverManager.getConnection(dbUrl, dbUser, dbPassword).use { connection ->
            connection.prepareStatement(transactionRequestQuery).executeQuery().use {
                transactionList = mapToObject(it)
            }
        }

        return transactionList
    }

    private fun loadProp(): Properties {

        val props = javaClass.classLoader.getResourceAsStream("application.properties").use {
            Properties().apply { load(it) }
        }
        return props
    }

    private fun getProp(prop: Properties, key: String): String {
        return prop.getProperty(key) ?: throw RuntimeException("property $prop not found")
    }

    private fun mapToObject(rs: ResultSet): List<TransactionRequest> {
        val list = mutableListOf<TransactionRequest>()
        while (rs.next()) {
            val transactionRequest = TransactionRequest(
                transactionNumber = rs.getString("txn_no"),
                pacsCurrentAccountNumber = rs.getString("src_ac_no"),
                neftBeneficiaryAccountNumber = rs.getString("dest_ac_no"),
                ifscCode = rs.getString("ifsc_code"),
                amount = rs.getString("txn_amt").toFloat(),
                date = rs.getDate("txn_date").toLocalDate(),
                tellerId = rs.getString("teller_id"),
                status = rs.getString("status"),
                beneficiaryName = rs.getString("beneficiary_name"),
                beneficiaryAddress = rs.getString("beneficiary_add"),
                pacsId = rs.getString("pacs_id"),
                dccbCode = rs.getString("dccb_code").padStart(4, '0'),
                branchCode = rs.getString("br_Code").padStart(3, '0'),
                remitterName = rs.getString("remitter_name"),
                pacsAccountNumber = rs.getString("pacs_acc_no"),
                linkedCBSAccountNumber = rs.getString("cbs_sb_acc_no"),
            )
            list.add(transactionRequest)
        }
        return list
    }
}